@@ -1,11 +1,16 @@
-use std::cmp::Ordering;
-use std::ops::RangeBounds;
+use core::cmp::Ordering;
+use core::mem::size_of;
+use core::ops::RangeBounds;
 
+use crate::bitmap::store::BITMAP_LENGTH;
 use crate::RoaringBitmap;
 
 use super::container::Container;
 use super::util;
 
+#[cfg(not(feature = "std"))]
+use alloc::vec::Vec;
+
 impl RoaringBitmap {
     /// Creates an empty `RoaringBitmap`.
     ///
@@ -31,6 +36,139 @@ impl RoaringBitmap {
         RoaringBitmap { containers: (0..=u16::MAX).map(Container::full).collect() }
     }
 
+    /// Creates a `RoaringBitmap` from a byte slice, interpreting the bytes as a bitmap with a specified offset.
+    ///
+    /// # Arguments
+    ///
+    /// - `offset: u32` - The starting position in the bitmap where the byte slice will be applied, specified in bits.
+    ///                   This means that if `offset` is `n`, the first byte in the slice will correspond to the `n`th bit(0-indexed) in the bitmap.
+    /// - `bytes: &[u8]` - The byte slice containing the bitmap data. The bytes are interpreted in "Least-Significant-First" bit order.
+    ///
+    /// # Interpretation of `bytes`
+    ///
+    /// The `bytes` slice is interpreted in "Least-Significant-First" bit order. Each byte is read from least significant bit (LSB) to most significant bit (MSB).
+    /// For example, the byte `0b00000101` represents the bits `1, 0, 1, 0, 0, 0, 0, 0` in that order (see Examples section).
+    ///
+    ///
+    /// # Panics
+    ///
+    /// This function will panic if `bytes.len() + offset` is greater than 2^32.
+    ///
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// use roaring::RoaringBitmap;
+    ///
+    /// let bytes = [0b00000101, 0b00000010, 0b00000000, 0b10000000];
+    /// //             ^^^^^^^^    ^^^^^^^^    ^^^^^^^^    ^^^^^^^^
+    /// //             76543210          98
+    /// let rb = RoaringBitmap::from_lsb0_bytes(0, &bytes);
+    /// assert!(rb.contains(0));
+    /// assert!(!rb.contains(1));
+    /// assert!(rb.contains(2));
+    /// assert!(rb.contains(9));
+    /// assert!(rb.contains(31));
+    ///
+    /// let rb = RoaringBitmap::from_lsb0_bytes(8, &bytes);
+    /// assert!(rb.contains(8));
+    /// assert!(!rb.contains(9));
+    /// assert!(rb.contains(10));
+    /// assert!(rb.contains(17));
+    /// assert!(rb.contains(39));
+    ///
+    /// let rb = RoaringBitmap::from_lsb0_bytes(3, &bytes);
+    /// assert!(rb.contains(3));
+    /// assert!(!rb.contains(4));
+    /// assert!(rb.contains(5));
+    /// assert!(rb.contains(12));
+    /// assert!(rb.contains(34));
+    /// ```
+    pub fn from_lsb0_bytes(offset: u32, mut bytes: &[u8]) -> RoaringBitmap {
+        fn shift_bytes(bytes: &[u8], amount: usize) -> Vec<u8> {
+            let mut result = Vec::with_capacity(bytes.len() + 1);
+            let mut carry = 0u8;
+
+            for &byte in bytes {
+                let shifted = (byte << amount) | carry;
+                carry = byte >> (8 - amount);
+                result.push(shifted);
+            }
+
+            if carry != 0 {
+                result.push(carry);
+            }
+
+            result
+        }
+        if offset % 8 != 0 {
+            let shift = offset as usize % 8;
+            let shifted_bytes = shift_bytes(bytes, shift);
+            return RoaringBitmap::from_lsb0_bytes(offset - shift as u32, &shifted_bytes);
+        }
+
+        if bytes.is_empty() {
+            return RoaringBitmap::new();
+        }
+
+        // Using inclusive range avoids overflow: the max exclusive value is 2^32 (u32::MAX + 1).
+        let end_bit_inc = u32::try_from(bytes.len())
+            .ok()
+            .and_then(|len_bytes| len_bytes.checked_mul(8))
+            // `bytes` is non-empty, so len_bits is > 0
+            .and_then(|len_bits| offset.checked_add(len_bits - 1))
+            .expect("offset + bytes.len() must be <= 2^32");
+
+        // offsets are in bytes
+        let (mut start_container, start_offset) =
+            (offset as usize >> 16, (offset as usize % 0x1_0000) / 8);
+        let (end_container_inc, end_offset) =
+            (end_bit_inc as usize >> 16, (end_bit_inc as usize % 0x1_0000 + 1) / 8);
+
+        let n_containers_needed = end_container_inc + 1 - start_container;
+        let mut containers = Vec::with_capacity(n_containers_needed);
+
+        // Handle a partial first container
+        if start_offset != 0 {
+            let end_byte = if end_container_inc == start_container {
+                end_offset
+            } else {
+                BITMAP_LENGTH * size_of::<u64>()
+            };
+
+            let (src, rest) = bytes.split_at(end_byte - start_offset);
+            bytes = rest;
+
+            if let Some(container) =
+                Container::from_lsb0_bytes(start_container as u16, src, start_offset)
+            {
+                containers.push(container);
+            }
+
+            start_container += 1;
+        }
+
+        // Handle all full containers
+        for full_container_key in start_container..end_container_inc {
+            let (src, rest) = bytes.split_at(BITMAP_LENGTH * size_of::<u64>());
+            bytes = rest;
+
+            if let Some(container) = Container::from_lsb0_bytes(full_container_key as u16, src, 0) {
+                containers.push(container);
+            }
+        }
+
+        // Handle a last container
+        if !bytes.is_empty() {
+            if let Some(container) = Container::from_lsb0_bytes(end_container_inc as u16, bytes, 0)
+            {
+                containers.push(container);
+            }
+        }
+
+        RoaringBitmap { containers }
+    }
+
     /// Adds a value to the set.
     ///
     /// Returns whether the value was absent from the set.
@@ -45,6 +183,7 @@ impl RoaringBitmap {
     /// assert_eq!(rb.insert(3), false);
     /// assert_eq!(rb.contains(3), true);
     /// ```
+    #[inline]
     pub fn insert(&mut self, value: u32) -> bool {
         let (key, index) = util::split(value);
         let container = match self.containers.binary_search_by_key(&key, |c| c.key) {
@@ -57,11 +196,12 @@ impl RoaringBitmap {
         container.insert(index)
     }
 
-    /// Search for the specific container by the given key.
-    /// Create a new container if not exist.
+    /// Searches for the specific container by the given key.
+    /// Creates a new container if it doesn't exist.
     ///
     /// Return the index of the target container.
-    fn find_container_by_key(&mut self, key: u16) -> usize {
+    #[inline]
+    pub(crate) fn find_container_by_key(&mut self, key: u16) -> usize {
         match self.containers.binary_search_by_key(&key, |c| c.key) {
             Ok(loc) => loc,
             Err(loc) => {
@@ -85,13 +225,14 @@ impl RoaringBitmap {
     /// assert!(rb.contains(3));
     /// assert!(!rb.contains(4));
     /// ```
+    #[inline]
     pub fn insert_range<R>(&mut self, range: R) -> u64
     where
         R: RangeBounds<u32>,
     {
         let (start, end) = match util::convert_range_to_inclusive(range) {
-            Some(range) => (*range.start(), *range.end()),
-            None => return 0,
+            Ok(range) => (*range.start(), *range.end()),
+            Err(_) => return 0,
         };
 
         let (start_container_key, start_index) = util::split(start);
@@ -149,6 +290,7 @@ impl RoaringBitmap {
     ///
     /// assert_eq!(rb.iter().collect::<Vec<u32>>(), vec![1, 3, 5]);
     /// ```
+    #[inline]
     pub fn push(&mut self, value: u32) -> bool {
         let (key, index) = util::split(value);
 
@@ -171,6 +313,7 @@ impl RoaringBitmap {
     /// # Panics
     ///
     /// If debug_assertions enabled and index is > self.max()
+    #[inline]
     pub(crate) fn push_unchecked(&mut self, value: u32) {
         let (key, index) = util::split(value);
 
@@ -200,12 +343,13 @@ impl RoaringBitmap {
     /// assert_eq!(rb.remove(3), false);
     /// assert_eq!(rb.contains(3), false);
     /// ```
+    #[inline]
     pub fn remove(&mut self, value: u32) -> bool {
         let (key, index) = util::split(value);
         match self.containers.binary_search_by_key(&key, |c| c.key) {
             Ok(loc) => {
                 if self.containers[loc].remove(index) {
-                    if self.containers[loc].len() == 0 {
+                    if self.containers[loc].is_empty() {
                         self.containers.remove(loc);
                     }
                     true
@@ -230,13 +374,14 @@ impl RoaringBitmap {
     /// rb.insert(3);
     /// assert_eq!(rb.remove_range(2..4), 2);
     /// ```
+    #[inline]
     pub fn remove_range<R>(&mut self, range: R) -> u64
     where
         R: RangeBounds<u32>,
     {
         let (start, end) = match util::convert_range_to_inclusive(range) {
-            Some(range) => (*range.start(), *range.end()),
-            None => return 0,
+            Ok(range) => (*range.start(), *range.end()),
+            Err(_) => return 0,
         };
 
         let (start_container_key, start_index) = util::split(start);
@@ -250,7 +395,7 @@ impl RoaringBitmap {
                 let a = if key == start_container_key { start_index } else { 0 };
                 let b = if key == end_container_key { end_index } else { u16::MAX };
                 removed += self.containers[index].remove_range(a..=b);
-                if self.containers[index].len() == 0 {
+                if self.containers[index].is_empty() {
                     self.containers.remove(index);
                     continue;
                 }
@@ -273,6 +418,7 @@ impl RoaringBitmap {
     /// assert_eq!(rb.contains(1), true);
     /// assert_eq!(rb.contains(100), false);
     /// ```
+    #[inline]
     pub fn contains(&self, value: u32) -> bool {
         let (key, index) = util::split(value);
         match self.containers.binary_search_by_key(&key, |c| c.key) {
@@ -300,14 +446,15 @@ impl RoaringBitmap {
     /// // 0xFFF is not contained
     /// assert!(!rb.contains_range(1..=0xFFF));
     /// ```
+    #[inline]
     pub fn contains_range<R>(&self, range: R) -> bool
     where
         R: RangeBounds<u32>,
     {
         let (start, end) = match util::convert_range_to_inclusive(range) {
-            Some(range) => (*range.start(), *range.end()),
-            // Empty ranges are always contained
-            None => return true,
+            Ok(range) => (*range.start(), *range.end()),
+            // Empty/Invalid ranges are always contained
+            Err(_) => return true,
         };
         let (start_high, start_low) = util::split(start);
         let (end_high, end_low) = util::split(end);
@@ -360,14 +507,15 @@ impl RoaringBitmap {
     /// assert_eq!(rb.range_cardinality(0x10000..0x10000), 0);
     /// assert_eq!(rb.range_cardinality(0x50000..=u32::MAX), 3);
     /// ```
+    #[inline]
     pub fn range_cardinality<R>(&self, range: R) -> u64
     where
         R: RangeBounds<u32>,
     {
         let (start, end) = match util::convert_range_to_inclusive(range) {
-            Some(range) => (*range.start(), *range.end()),
-            // Empty ranges have 0 bits set in them
-            None => return 0,
+            Ok(range) => (*range.start(), *range.end()),
+            // Empty/invalid ranges have 0 bits set in them
+            Err(_) => return 0,
         };
 
         let (start_key, start_low) = util::split(start);
@@ -419,6 +567,7 @@ impl RoaringBitmap {
     /// rb.clear();
     /// assert_eq!(rb.contains(1), false);
     /// ```
+    #[inline]
     pub fn clear(&mut self) {
         self.containers.clear();
     }
@@ -436,6 +585,7 @@ impl RoaringBitmap {
     /// rb.insert(3);
     /// assert_eq!(rb.is_empty(), false);
     /// ```
+    #[inline]
     pub fn is_empty(&self) -> bool {
         self.containers.is_empty()
     }
@@ -451,6 +601,7 @@ impl RoaringBitmap {
     /// assert!(!rb.is_empty());
     /// assert!(rb.is_full());
     /// ```
+    #[inline]
     pub fn is_full(&self) -> bool {
         self.containers.len() == (u16::MAX as usize + 1)
             && self.containers.iter().all(Container::is_full)
@@ -473,6 +624,7 @@ impl RoaringBitmap {
     /// rb.insert(4);
     /// assert_eq!(rb.len(), 2);
     /// ```
+    #[inline]
     pub fn len(&self) -> u64 {
         self.containers.iter().map(|container| container.len()).sum()
     }
@@ -491,6 +643,7 @@ impl RoaringBitmap {
     /// rb.insert(4);
     /// assert_eq!(rb.min(), Some(3));
     /// ```
+    #[inline]
     pub fn min(&self) -> Option<u32> {
         self.containers.first().and_then(|tail| tail.min().map(|min| util::join(tail.key, min)))
     }
@@ -509,6 +662,7 @@ impl RoaringBitmap {
     /// rb.insert(4);
     /// assert_eq!(rb.max(), Some(4));
     /// ```
+    #[inline]
     pub fn max(&self) -> Option<u32> {
         self.containers.last().and_then(|tail| tail.max().map(|max| util::join(tail.key, max)))
     }
@@ -528,6 +682,7 @@ impl RoaringBitmap {
     /// assert_eq!(rb.rank(3), 1);
     /// assert_eq!(rb.rank(10), 2)
     /// ```
+    #[inline]
     pub fn rank(&self, value: u32) -> u64 {
         // if len becomes cached for RoaringBitmap: return len if len > value
 
@@ -562,6 +717,7 @@ impl RoaringBitmap {
     /// assert_eq!(rb.select(2), Some(100));
     /// assert_eq!(rb.select(3), None);
     /// ```
+    #[inline]
     pub fn select(&self, n: u32) -> Option<u32> {
         let mut n = n as u64;
 
@@ -593,6 +749,7 @@ impl RoaringBitmap {
     /// let mut rb = RoaringBitmap::from_iter([1, 3, 7, 9]);
     /// rb.remove_smallest(2);
     /// assert_eq!(rb, RoaringBitmap::from_iter([7, 9]));
+    #[inline]
     pub fn remove_smallest(&mut self, mut n: u64) {
         // remove containers up to the front of the target
         let position = self.containers.iter().position(|container| {
@@ -627,6 +784,7 @@ impl RoaringBitmap {
     /// assert_eq!(rb, RoaringBitmap::from_iter([1, 5]));
     /// rb.remove_biggest(1);
     /// assert_eq!(rb, RoaringBitmap::from_iter([1]));
+    #[inline]
     pub fn remove_biggest(&mut self, mut n: u64) {
         // remove containers up to the back of the target
         let position = self.containers.iter().rposition(|container| {
@@ -690,7 +848,7 @@ mod tests {
 
             // Assert all values in the range are present
             for i in r.clone() {
-                assert!(b.contains(i), "does not contain {}", i);
+                assert!(b.contains(i), "does not contain {i}");
             }
 
             // Run the check values looking for any false positives
@@ -699,8 +857,7 @@ mod tests {
                 let range_has = r.contains(&i);
                 assert_eq!(
                     bitmap_has, range_has,
-                    "value {} in bitmap={} and range={}",
-                    i, bitmap_has, range_has
+                    "value {i} in bitmap={bitmap_has} and range={range_has}"
                 );
             }
         }
