@@ -3,6 +3,7 @@ mod container;
 mod fmt;
 mod multiops;
 mod proptests;
+mod statistics;
 mod store;
 mod util;
 
@@ -12,13 +13,20 @@ mod cmp;
 mod inherent;
 mod iter;
 mod ops;
+#[cfg(feature = "std")]
+mod ops_with_serialized;
 #[cfg(feature = "serde")]
 mod serde;
+#[cfg(feature = "std")]
 mod serialization;
 
 use self::cmp::Pairs;
 pub use self::iter::IntoIter;
 pub use self::iter::Iter;
+pub use self::statistics::Statistics;
+
+#[cfg(not(feature = "std"))]
+use alloc::vec::Vec;
 
 /// A compressed bitmap using the [Roaring bitmap compression scheme](https://roaringbitmap.org/).
 ///
