@@ -1,5 +1,5 @@
 use crate::RoaringBitmap;
-use std::collections::BTreeMap;
+use alloc::collections::BTreeMap;
 
 mod fmt;
 mod multiops;
@@ -14,9 +14,10 @@ mod iter;
 mod ops;
 #[cfg(feature = "serde")]
 mod serde;
+#[cfg(feature = "std")]
 mod serialization;
 
-pub use self::iter::{IntoIter, Iter};
+pub use self::iter::{BitmapIter, IntoIter, Iter};
 
 /// A compressed bitmap with u64 values.
 /// Implemented as a `BTreeMap` of `RoaringBitmap`s.
