@@ -1,5 +1,6 @@
+#![cfg(feature = "std")]
+
 use roaring::RoaringTreemap;
-use std::iter::FromIterator;
 
 fn serialize_deserialize<Dataset, I>(dataset: Dataset)
 where
